///////////////////////////////////////////////////////////////
//                                                           //
// dirNav.cpp is based on Dr. Fawcett's dir2.cpp             //
//                                                           //
// dirNav.cpp - traverse the directories under the current   //
//            directory. In each directory, invoke the       // 
//            function specified by the function pointer     //
//            in the argument list                           //
//                                                           //
///////////////////////////////////////////////////////////////
//                                                           //
// Adapted by Jim Fawcett from the book:                     //
// "Win32 System Services:                                   //
// The Heart of Windows 95 and Windows NT", Marshall Brain,  //
// Prentice Hall, Copyright 1995                             //
//                                                           //
///////////////////////////////////////////////////////////////
// Note: fixed bug in published code which fails to report   //
//       directory change to parent, associating files with  //
//       incorrect directory                                 //
///////////////////////////////////////////////////////////////

#include "dirNav.h"

//----< Recursively traverse directories >------------------------

// void dirNav::traverse(char *dirName, char *fileMask, void (*perform)())
// void dirNav::traverse( const char *dirName, const char *fileMask, const bool autoDoInThisDir )
void dirNav::traverse( const char *dirName, const char *fileMask )
{
  char *fileName;
  char curDir[ 256 ];
  char fullName[ 256 ];
  char prevFullName[256];
  HANDLE fileHandle;
  WIN32_FIND_DATA findData;
  
  // save current dir so it can restore it
  if( !GetCurrentDirectory( 256, curDir) ) 
    return;

  // if the directory name is neither . or .. then
  // change to it, otherwise ignore it
  if( strcmp( dirName, "." ) && strcmp( dirName, ".." ) ) {
    if( !SetCurrentDirectory( dirName ) ) return;
  } else {
    return;
  }

  // To check if we can take the action in current directory
  // userAnswer = getUserAnswer( autoDoInThisDir );

  // If the navigator gets the current directory name correctly,
  // it visit this directory
  enterDir();

  // To decide if we can automatically take the action
  // for all the subdirectories
  // autoDoForSubDir = ( userAnswer == 'A' ? true : false );

  // print out the current directory name
  // Don't know what the following statement is for.
  strncpy(prevFullName,fullName,256);

  if( !GetFullPathName( fileMask, 256, fullName, &fileName ) ) 
    return;
  if( strcmp( fullName,prevFullName ) != 0 )
    cout << "  " << fullName << endl;

  // Loop through all files in the directory
  fileHandle = FindFirstFile( fileMask, &findData );

  while ( fileHandle != INVALID_HANDLE_VALUE ) {
	  if( findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ) {
		  // If it is a directory, traverse it recursively
		  // traverse( findData.cFileName, fileMask, autoDoForSubDir );
		  traverse( findData.cFileName, fileMask );
	  } else {
		  // Otherwise take the action on the file
		  // performF( findData.cFileName, userAnswer );
		  processFile( findData.cFileName );
	  }

    // loop thru remaining entries in the dir
	  if (!FindNextFile( fileHandle, &findData ))
		  break;
  }

  leaveDir();

  // clean up and restore directory
  FindClose( fileHandle );
  SetCurrentDirectory( curDir );

  // print out the current directory name
  strncpy(prevFullName,fullName,256);
  if( !GetFullPathName( fileMask, 256, fullName, &fileName ) ) 
    return;
  if(strcmp(prevFullName,fullName) != 0)
    cout << "  " << fullName << endl;
}

//

//----< test stub >--------------------------------------------

#ifdef NAV_TEST

class testNav : public dirNav{
public:
	char getUserAnswer( const bool confirm )
	{
		char answer;
		if ( confirm ) {
		// If automatically delete files, return 'A'
			return 'A';
		} else {
			do {
				cout << "\nYou wanna delete all the non-essential files in the currnt directory?\n" << endl;
				cout << "Please enter:\n";
				cout << "(Y) to delete the files in this directory. " << endl;
				cout << "(N) to skip this directory" << endl;
				cout << "(A) to delete all the nonessntial files" << endl;
				cout << "    in this directory and its subdirectories" << endl;
				cin >> answer;
			} while (answer != 'Y' && answer != 'y'
				&& answer != 'N' && answer != 'n'
				&& answer != 'A' && answer != 'a');
			if (answer == 'A' || answer == 'a') {
				return 'A';
			} else if (answer == 'Y' || answer == 'y') {
				return 'Y';
			} else {
				return 'N';
			}
		}
	}

	void performF(const char * fName, const char userAnswer)
	{
		cout << "User's answer is: " << userAnswer << " for " << fName << endl;
	}
};

int main(int argc, char *argv[])
{
  testNav * visitor = new testNav;
  cout << "\n  Demonstrating Directory Traversal "
       << "\n ===================================\n\n";

  char curDir[ 256 ];

  if( !GetCurrentDirectory( 256, curDir ) ) {
    cerr << "Couldn't get the current directory." << endl;
    return( 1 );
  }

  visitor->traverse(curDir, "*.*", false);
  cout << endl;

  // List all files, starting with the current directory
  // visitor->traverse(curDir, "*.*", true);
  // cout << endl;

  return( 0 );
}

//----< prints information about a file >----------------------

/* void PrintFindData(WIN32_FIND_DATA *findData)

{	
  cout << setw(20) << findData->cFileName;
  cout << setw(10) << findData->nFileSizeLow << endl;
} */

#endif