#ifndef DIRNAV_H
#define DIRNAV_H

///////////////////////////////////////////////////////////////
// dirNav.h - traverse subtree below specified Directory     //
//                                                           //
// Modified By Qiang Zheng From Dr. Fawcett's example.		 //
// Apr. 08 1999												 //
//															 //
// Adapted by Jim Fawcett from the book:                     //
// "Win32 System Services:                                   //
// The Heart of Windows 95 and Windows NT", Marshall Brain,  //
// Prentice Hall, Copyright 1995                             //
//                                                           //
///////////////////////////////////////////////////////////////
// Note: fixed bug in published code which fails to report   //
//       directory change to parent, associating files with  //
//       incorrect directory                                 //
///////////////////////////////////////////////////////////////

#include <iostream>
#include <iomanip>
#include <cstring> 
#include <windows.h>
using namespace std;

// This virtual class is the base class of required "CLEAN" and "DUPS"
// class, i.e. it is a public interface	
class dirNav {
public:
	// The actions will be done when navigator enters a new directory 
	// or leaves this directory
	virtual void enterDir( ) = 0;
	virtual void leaveDir( ) = 0;

	// When navigator visits a file, it can process this file.
	virtual void processFile( const char * fName ) = 0;

	// Recursively traverses subdirectories
	// void traverse(char *dirName, char *fileMask, void (*perform) () );
	void traverse( const char *dirName, const char *fileMask );

	// prints information about a file
	// void PrintFindData(WIN32_FIND_DATA *findData);

	// in the current directory and its subdirectories
	// virtual char getUserAnswer( const bool confirm ) = 0;

	// Special actions to be performed when files are encountered
	// this virtual function has to be implemented by the derived class
	// virtual void processFile( const char * fName, const char userAnswer ) = 0;


};

#endif